/*
	 Copyright (C) 2003 Paul Brossier

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.

	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.

	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	 
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <getopt.h>
#include <unistd.h>

#include "aubio.h"	

#include "utils.h"


const char * output_filename = NULL;
const char * input_filename = NULL;

int verbose = 0;
int usejack = 0;

int aubio_process(float **input, float **output, int nframes);

/* settings */
int buffer_size = 1024;
int overlap_size = 512;
int channels = 2;

/* global objects */
aubio_pvoc_t * pv;

fvec_t *ibuf;
fvec_t *obuf;
cvec_t *fftgrain;

int aubio_process(float **input, float **output, int nframes) {
	unsigned int i; 			/*channels*/
	unsigned int j; 			/*frames*/   
	unsigned int pos = 0; /*frames%dspblocksize*/
	//unsigned int norm = 0; /*norm counter*/
	for (j=0;j<nframes;j++) {
		for (i=0;i<channels;i++) { 
			/* write input to datanew */
			fvec_write_sample(ibuf, input[i][j], i, pos);
			/* put synthnew in output */
			output[i][j] = fvec_read_sample(obuf, i, pos);
		}
		/*time for fft*/
		if (pos == overlap_size-1) { 			
			aubio_pvoc_do (pv,ibuf, fftgrain);
			for (i=0;i<channels;i++) {
				for (j=0;j<fftgrain->length;j++) {
					fftgrain->phas[i][j] = 0.;
				}
			}
			aubio_pvoc_rdo(pv,fftgrain, obuf);
			pos = -1; /* so it will be zero next j loop */
		}
		pos++;
	}
	return 1;
}

int main (int argc, char **argv) {  
	int frames;

	aubio_file_t * file = NULL;
	aubio_file_t * fileout = NULL;

	parse_args(argc, argv);
	
	if(!usejack) 
	{
		debug("Opening  files ...\n");
		file = new_file_ro (input_filename);
		file_info(file);
		channels = file_channels(file);
		fileout = new_file_wo(file, output_filename);
	}

	ibuf = new_fvec(overlap_size, channels);
	obuf = new_fvec(overlap_size, channels);
	fftgrain = new_cvec(buffer_size, channels);
	
	/* phase vocoder */
	debug("Phase voc init ... \n");
	pv = new_aubio_pvoc(buffer_size, overlap_size, channels);
	
	// command line argument parsing
	if(usejack) {
		aubio_jack_t * jack_setup;

		debug("Jack init ...\n");
		jack_setup = new_aubio_jack(channels, channels, (aubio_process_func_t)aubio_process);

		debug("Jack activation ...\n");
		aubio_jack_activate(jack_setup);

		debug("Processing (Ctrl+C to quit) ...\n");
		pause();

		aubio_jack_close(jack_setup);

	} else {

		fvec_t * inputbuf ;
		fvec_t * outputbuf;
		/* file objects */

		inputbuf 	= new_fvec(overlap_size, channels);
		outputbuf	= new_fvec(overlap_size, channels);	
		/* phasevoc init */
		debug("Processing ...\n");
		/* reading and processing data from file */
		debug("coucou\n");
		frames = 0;
		while ((overlap_size == file_read(file, overlap_size, inputbuf)) ) {
			// if end of file, fill with zeros
			frames++;
			aubio_process(inputbuf->data, outputbuf->data, overlap_size);
			// write output to file (but not the first frame)
			if ((output_filename != NULL) && (frames != 1) )
				file_write(fileout,overlap_size,outputbuf);
		}
		debug("Processed %d frames of %d samples.\n", frames, buffer_size);

		del_file(file);
		del_file(fileout);

	}

	del_aubio_pvoc(pv);

	del_fvec(obuf);
	del_fvec(ibuf);
	del_cvec(fftgrain);
	
	debug("End of program.\n");
	fflush(stderr);
	return 0;
}

