
#include "aubio.h"

#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include "utils.h"


const char * prog_name;

void usage (FILE * stream, int exit_code)
{
	fprintf(stream, "usage: %s [ options ] \n", prog_name);
	fprintf(stream, 
			"	-j	--jack		Use Jack.\n"
			"	-o	--output	Output type.\n"
			"	-i	--input		Input type.\n"
			"	-h	--help		Display this message.\n"
			"	-v	--verbose	Print verbose message.\n"
			);
	exit(exit_code);
}

int parse_args (int argc, char **argv) {
	const char *options = "hvjo:i:";
	int next_option;
	struct option long_options[] =
	{
		{"help", 		0, NULL, 'h'},
		{"verbose",	0, NULL, 'v'},
		{"jack", 		0, NULL, 'j'},
		{"output", 	0, NULL, 'o'},
		{"input", 	0, NULL, 'i'},
		{NULL,			0, NULL, 0}
	};
	prog_name = argv[0];	
	if( argc < 1 ) {
		usage (stderr, 1);
		return -1;
	}
	do {
		next_option = getopt_long (argc, argv, options, 
									long_options, NULL);
		switch (next_option) {
			case 'o':
				output_filename = optarg;
				break;
			case 'i':
				input_filename = optarg;
				break;
			case 'h': 	/* help */
				usage (stdout, 0);
				return -1;
			case 'v':		/* verbose */
				verbose = 1;
				break;
			case 'j':		/* verbose */
				usejack = 1;
				break;
			case '?': 	/* unknown options */
				usage(stderr, 1);
			case -1: 		/* done with options */
				break;
			default: 		/*something else unexpected */
				abort ();
		}
	}
	while (next_option != -1);

	if (output_filename != NULL && input_filename != NULL) {
		printf ("Output file : %s\n", output_filename );
		printf ("Input file : %s\n", input_filename );
	} else {
		printf ("Jack input output\n");
		usejack = 1;
	}	
	return 0;
}

