/*
	 Copyright (C) 2003 Paul Brossier

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.

	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.

	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	 */
#include "aubio_priv.h"
#include "sample.h"

static fvec_t * fvec_malloc(int length, int channels);
static cvec_t * cvec_malloc(int length, int channels);

fvec_t * new_fvec( int length, int channels) {
	fvec_t * buf = fvec_malloc(length,channels);
	return buf;
}

fvec_t * fvec_malloc(int length, int channels) {
	// actual structure allocation
	fvec_t * s = (fvec_t *) malloc(sizeof(fvec_t));
	unsigned int i;
	s->channels = channels;
	s->length = length;
	s->data = AUBIO_ARRAY(float*,s->channels);
	for (i=0; i< s->channels; i++) {
		s->data[i] = AUBIO_ARRAY(float, s->length);
		//AUBIO_MEMSET(s->data[i],0,float);
	}
	return s;
}

void del_fvec(fvec_t *s) {
	unsigned int i;
	for (i=0; i<s->channels; i++) {
		AUBIO_FREE(s->data[i]);
	}
	AUBIO_FREE(s->data);
	AUBIO_FREE(s);
}

void fvec_write_sample(fvec_t *s, float data, int channel, int position) {
	s->data[channel][position] = data;
}
float fvec_read_sample(fvec_t *s, int channel, int position) {
	return s->data[channel][position];
}
void fvec_put_channel(fvec_t *s, float * data, int channel) {
	s->data[channel] = data;
}
float * fvec_get_channel(fvec_t *s, int channel) {
	return s->data[channel];
}

cvec_t * new_cvec( int length, int channels) {
	cvec_t * buf = cvec_malloc(length,channels);
	return buf;
}

cvec_t * cvec_malloc(int length, int channels) {
	// actual structure allocation
	cvec_t * s = AUBIO_NEW(cvec_t);
	unsigned int i;
	s->channels = channels;
	s->length = length;
	//s->data = AUBIO_ARRAY(fft_data_t*,s->channels);
	s->norm = AUBIO_ARRAY(float*,s->channels);
	s->phas = AUBIO_ARRAY(float*,s->channels);
	for (i=0; i< s->channels; i++) {
		//s->data[i] = AUBIO_ARRAY(fft_data_t,s->length);
		s->norm[i] = AUBIO_ARRAY(float,s->length);
		s->phas[i] = AUBIO_ARRAY(float,s->length);
	}
	return s;
}

void del_cvec(cvec_t *s) {
	unsigned int i;
	for (i=0; i<s->channels; i++) {
		//AUBIO_FREE(s->data[i]);
		AUBIO_FREE(s->norm[i]);
		AUBIO_FREE(s->phas[i]);
	}
	//AUBIO_FREE(s->data);
	AUBIO_FREE(s->norm);
	AUBIO_FREE(s->phas);
	AUBIO_FREE(s);
}
